@@ -28,26 +28,26 @@ enum {
 
 G_DEFINE_QUARK (retro-core-descriptor-error, retro_core_descriptor_error)
 
-#define RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP "Libretro"
-#define RETRO_CORE_DESCRIPTOR_PLATFORM_GROUP_PREFIX "Platform:"
-#define RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX "Firmware:"
+#define LIBRETRO_GROUP "Libretro"
+#define PLATFORM_GROUP_PREFIX "Platform:"
+#define FIRMWARE_GROUP_PREFIX "Firmware:"
 
-#define RETRO_CORE_DESCRIPTOR_TYPE_KEY "Type"
-#define RETRO_CORE_DESCRIPTOR_NAME_KEY "Name"
-#define RETRO_CORE_DESCRIPTOR_ICON_KEY "Icon"
-#define RETRO_CORE_DESCRIPTOR_MODULE_KEY "Module"
-#define RETRO_CORE_DESCRIPTOR_LIBRETRO_VERSION_KEY "LibretroVersion"
+#define TYPE_KEY "Type"
+#define NAME_KEY "Name"
+#define ICON_KEY "Icon"
+#define MODULE_KEY "Module"
+#define LIBRETRO_VERSION_KEY "LibretroVersion"
 
-#define RETRO_CORE_DESCRIPTOR_PLATFORM_MIME_TYPE_KEY "MimeType"
-#define RETRO_CORE_DESCRIPTOR_PLATFORM_FIRMWARES_KEY "Firmwares"
+#define PLATFORM_MIME_TYPE_KEY "MimeType"
+#define PLATFORM_FIRMWARES_KEY "Firmwares"
 
-#define RETRO_CORE_DESCRIPTOR_FIRMWARE_PATH_KEY "Path"
-#define RETRO_CORE_DESCRIPTOR_FIRMWARE_MD5_KEY "MD5"
-#define RETRO_CORE_DESCRIPTOR_FIRMWARE_SHA512_KEY "SHA-512"
-#define RETRO_CORE_DESCRIPTOR_FIRMWARE_MANDATORY_KEY "Mandatory"
+#define FIRMWARE_PATH_KEY "Path"
+#define FIRMWARE_MD5_KEY "MD5"
+#define FIRMWARE_SHA512_KEY "SHA-512"
+#define FIRMWARE_MANDATORY_KEY "Mandatory"
 
-#define RETRO_CORE_DESCRIPTOR_TYPE_GAME "Game"
-#define RETRO_CORE_DESCRIPTOR_TYPE_EMULATOR "Emulator"
+#define TYPE_GAME "Game"
+#define TYPE_EMULATOR "Emulator"
 
 /* Private */
 
@@ -80,17 +80,16 @@ has_group_prefixed (RetroCoreDescriptor *self,
                     const gchar         *group_prefix,
                     const gchar         *group_suffix)
 {
-  gchar *group;
-  gboolean result;
+  g_autofree gchar *group = NULL;
+  gboolean has_group;
 
   g_assert (group_prefix != NULL);
   g_assert (group_suffix != NULL);
 
   group = g_strconcat (group_prefix, group_suffix, NULL);
-  result = g_key_file_has_group (self->key_file, group);
-  g_free (group);
+  has_group = g_key_file_has_group (self->key_file, group);
 
-  return result;
+  return has_group;
 }
 
 static gboolean
@@ -100,8 +99,8 @@ has_key_prefixed (RetroCoreDescriptor  *self,
                   const gchar          *key,
                   GError              **error)
 {
-  gchar *group;
-  gboolean result;
+  g_autofree gchar *group = NULL;
+  gboolean has_key;
   GError *tmp_error = NULL;
 
   g_assert (group_prefix != NULL);
@@ -109,20 +108,17 @@ has_key_prefixed (RetroCoreDescriptor  *self,
   g_assert (key != NULL);
 
   group = g_strconcat (group_prefix, group_suffix, NULL);
-  result = g_key_file_has_key (self->key_file,
-                               group,
-                               key,
-                               &tmp_error);
+  has_key = g_key_file_has_key (self->key_file,
+                                group,
+                                key,
+                                &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
-    g_free (group);
     g_propagate_error (error, tmp_error);
 
     return FALSE;
   }
 
-  g_free (group);
-
-  return result;
+  return has_key;
 }
 
 static gchar *
@@ -132,8 +128,8 @@ get_string_prefixed (RetroCoreDescriptor  *self,
                      const gchar          *key,
                      GError              **error)
 {
-  gchar *group;
-  gchar *result;
+  g_autofree gchar *group = NULL;
+  g_autofree gchar *string = NULL;
   GError *tmp_error = NULL;
 
   g_assert (group_prefix != NULL);
@@ -142,20 +138,14 @@ get_string_prefixed (RetroCoreDescriptor  *self,
 
   group = g_strconcat (group_prefix, group_suffix, NULL);
 
-  result = g_key_file_get_string (self->key_file,
+  string = g_key_file_get_string (self->key_file,
                                   group,
                                   key,
                                   &tmp_error);
-  if (G_UNLIKELY (tmp_error != NULL)) {
-    g_free (group);
-    g_propagate_error (error, tmp_error);
-
+  if (G_UNLIKELY (tmp_error != NULL))
     return NULL;
-  }
 
-  g_free (group);
-
-  return result;
+  return g_steal_pointer (&string);
 }
 
 static gchar **
@@ -166,8 +156,8 @@ get_string_list_prefixed (RetroCoreDescriptor  *self,
                           gsize                *length,
                           GError              **error)
 {
-  gchar *group;
-  gchar **result;
+  g_autofree gchar *group = NULL;
+  g_auto (GStrv) list = NULL;
   GError *tmp_error = NULL;
 
   g_assert (group_prefix != NULL);
@@ -176,21 +166,18 @@ get_string_list_prefixed (RetroCoreDescriptor  *self,
   g_assert (length != NULL);
 
   group = g_strconcat (group_prefix, group_suffix, NULL);
-  result = g_key_file_get_string_list (self->key_file,
-                                       group,
-                                       key,
-                                       length,
-                                       &tmp_error);
+  list = g_key_file_get_string_list (self->key_file,
+                                     group,
+                                     key,
+                                     length,
+                                     &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
-  g_free (group);
     g_propagate_error (error, tmp_error);
 
     return NULL;
   }
 
-  g_free (group);
-
-  return result;
+  return g_steal_pointer (&list);
 }
 
 static void
@@ -206,8 +193,8 @@ check_has_required_key (RetroCoreDescriptor  *self,
   g_assert (key != NULL);
 
   has_key = g_key_file_has_key (self->key_file,
-                                RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                RETRO_CORE_DESCRIPTOR_TYPE_KEY,
+                                LIBRETRO_GROUP,
+                                TYPE_KEY,
                                 &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -233,8 +220,8 @@ check_libretro_group (RetroCoreDescriptor  *self,
   GError *tmp_error = NULL;
 
   check_has_required_key (self,
-                          RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                          RETRO_CORE_DESCRIPTOR_TYPE_KEY,
+                          LIBRETRO_GROUP,
+                          TYPE_KEY,
                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -243,8 +230,8 @@ check_libretro_group (RetroCoreDescriptor  *self,
   }
 
   check_has_required_key (self,
-                          RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                          RETRO_CORE_DESCRIPTOR_NAME_KEY,
+                          LIBRETRO_GROUP,
+                          NAME_KEY,
                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -253,8 +240,8 @@ check_libretro_group (RetroCoreDescriptor  *self,
   }
 
   check_has_required_key (self,
-                          RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                          RETRO_CORE_DESCRIPTOR_MODULE_KEY,
+                          LIBRETRO_GROUP,
+                          MODULE_KEY,
                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -263,8 +250,8 @@ check_libretro_group (RetroCoreDescriptor  *self,
   }
 
   check_has_required_key (self,
-                          RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                          RETRO_CORE_DESCRIPTOR_LIBRETRO_VERSION_KEY,
+                          LIBRETRO_GROUP,
+                          LIBRETRO_VERSION_KEY,
                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -279,21 +266,24 @@ check_platform_group (RetroCoreDescriptor  *self,
                       GError              **error)
 {
   gboolean has_key;
-  gchar **firmwares;
-  gchar **firmware_p;
-  gchar *firmware_group;
+  g_auto (GStrv) firmwares = NULL;
   GError *tmp_error = NULL;
 
   g_assert (group != NULL);
 
   check_has_required_key (self,
                           group,
-                          RETRO_CORE_DESCRIPTOR_PLATFORM_MIME_TYPE_KEY,
+                          PLATFORM_MIME_TYPE_KEY,
                           &tmp_error);
+  if (G_UNLIKELY (tmp_error != NULL)) {
+    g_propagate_error (error, tmp_error);
+
+    return;
+  }
 
   has_key = g_key_file_has_key (self->key_file,
                                 group,
-                                RETRO_CORE_DESCRIPTOR_PLATFORM_FIRMWARES_KEY,
+                                PLATFORM_FIRMWARES_KEY,
                                 &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -306,7 +296,7 @@ check_platform_group (RetroCoreDescriptor  *self,
 
   firmwares = g_key_file_get_string_list (self->key_file,
                                           group,
-                                          RETRO_CORE_DESCRIPTOR_PLATFORM_FIRMWARES_KEY,
+                                          PLATFORM_FIRMWARES_KEY,
                                           NULL,
                                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
@@ -315,8 +305,10 @@ check_platform_group (RetroCoreDescriptor  *self,
     return;
   }
 
-  for (firmware_p = firmwares; *firmware_p != NULL; firmware_p++) {
-    firmware_group = g_strconcat (RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX, *firmware_p, NULL);
+  for (GStrv firmware_p = firmwares; *firmware_p != NULL; firmware_p++) {
+    g_autofree gchar *firmware_group = NULL;
+
+    firmware_group = g_strconcat (FIRMWARE_GROUP_PREFIX, *firmware_p, NULL);
     if (!g_key_file_has_group (self->key_file, firmware_group)) {
       g_set_error (error,
                    RETRO_CORE_DESCRIPTOR_ERROR,
@@ -327,16 +319,9 @@ check_platform_group (RetroCoreDescriptor  *self,
                    firmware_group,
                    group);
 
-      g_free (firmware_group);
-      g_strfreev (firmwares);
-
       return;
     }
-
-    g_free (firmware_group);
   }
-
-  g_strfreev (firmwares);
 }
 
 static void
@@ -350,7 +335,7 @@ check_firmware_group (RetroCoreDescriptor  *self,
 
   check_has_required_key (self,
                           group,
-                          RETRO_CORE_DESCRIPTOR_FIRMWARE_PATH_KEY,
+                          FIRMWARE_PATH_KEY,
                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -360,7 +345,7 @@ check_firmware_group (RetroCoreDescriptor  *self,
 
   check_has_required_key (self,
                           group,
-                          RETRO_CORE_DESCRIPTOR_FIRMWARE_MANDATORY_KEY,
+                          FIRMWARE_MANDATORY_KEY,
                           &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -390,8 +375,8 @@ retro_core_descriptor_has_icon (RetroCoreDescriptor  *self,
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
 
   has_icon = g_key_file_has_key (self->key_file,
-                                 RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                 RETRO_CORE_DESCRIPTOR_ICON_KEY,
+                                 LIBRETRO_GROUP,
+                                 ICON_KEY,
                                  &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -413,16 +398,15 @@ retro_core_descriptor_has_icon (RetroCoreDescriptor  *self,
 gchar *
 retro_core_descriptor_get_uri (RetroCoreDescriptor *self)
 {
-  gchar *result;
-  GFile *file;
+  g_autoptr (GFile) file = NULL;
+  g_autofree gchar *uri = NULL;
 
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
 
   file = g_file_new_for_path (self->filename);
-  result = g_file_get_uri (file);
-  g_object_unref (file);
+  uri = g_file_get_uri (file);
 
-  return result;
+  return g_steal_pointer (&uri);
 }
 
 /**
@@ -454,24 +438,23 @@ gboolean
 retro_core_descriptor_get_is_game (RetroCoreDescriptor  *self,
                                    GError              **error)
 {
-  gchar *type;
+  g_autofree gchar *type = NULL;
   gboolean is_game;
   GError *tmp_error = NULL;
 
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
 
   type = g_key_file_get_string (self->key_file,
-                                  RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                  RETRO_CORE_DESCRIPTOR_TYPE_KEY,
-                                  &tmp_error);
+                                LIBRETRO_GROUP,
+                                TYPE_KEY,
+                                &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
 
     return FALSE;
   }
 
-  is_game = g_strcmp0 (type, RETRO_CORE_DESCRIPTOR_TYPE_GAME) == 0;
-  g_free (type);
+  is_game = g_strcmp0 (type, TYPE_GAME) == 0;
 
   return is_game;
 }
@@ -489,26 +472,25 @@ gboolean
 retro_core_descriptor_get_is_emulator (RetroCoreDescriptor  *self,
                                        GError              **error)
 {
-  gchar *type;
-  gboolean is_game;
+  g_autofree gchar *type = NULL;
+  gboolean is_emulator;
   GError *tmp_error = NULL;
 
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
 
   type = g_key_file_get_string (self->key_file,
-                                  RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                  RETRO_CORE_DESCRIPTOR_TYPE_KEY,
-                                  &tmp_error);
+                                LIBRETRO_GROUP,
+                                TYPE_KEY,
+                                &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
 
     return FALSE;
   }
 
-  is_game = g_strcmp0 (type, RETRO_CORE_DESCRIPTOR_TYPE_EMULATOR) == 0;
-  g_free (type);
+  is_emulator = g_strcmp0 (type, TYPE_EMULATOR) == 0;
 
-  return is_game;
+  return is_emulator;
 }
 
 /**
@@ -524,9 +506,11 @@ gchar *
 retro_core_descriptor_get_name (RetroCoreDescriptor  *self,
                                 GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
+
   return g_key_file_get_string (self->key_file,
-                                RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                RETRO_CORE_DESCRIPTOR_NAME_KEY,
+                                LIBRETRO_GROUP,
+                                NAME_KEY,
                                 error);
 }
 
@@ -543,15 +527,15 @@ GIcon *
 retro_core_descriptor_get_icon (RetroCoreDescriptor  *self,
                                 GError              **error)
 {
-  gchar *icon_name;
+  g_autofree gchar *icon_name = NULL;
   GIcon *icon;
   GError *tmp_error = NULL;
 
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
 
   icon_name = g_key_file_get_string (self->key_file,
-                                     RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                     RETRO_CORE_DESCRIPTOR_ICON_KEY,
+                                     LIBRETRO_GROUP,
+                                     ICON_KEY,
                                      &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
@@ -560,7 +544,6 @@ retro_core_descriptor_get_icon (RetroCoreDescriptor  *self,
   }
 
   icon = G_ICON (g_themed_icon_new (icon_name));
-  g_free (icon_name);
 
   return icon;
 }
@@ -581,8 +564,8 @@ retro_core_descriptor_get_module (RetroCoreDescriptor  *self,
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
 
   return g_key_file_get_string (self->key_file,
-                                RETRO_CORE_DESCRIPTOR_LIBRETRO_GROUP,
-                                RETRO_CORE_DESCRIPTOR_MODULE_KEY,
+                                LIBRETRO_GROUP,
+                                MODULE_KEY,
                                 error);
 }
 
@@ -599,41 +582,31 @@ GFile *
 retro_core_descriptor_get_module_file (RetroCoreDescriptor  *self,
                                        GError              **error)
 {
-  GFile *file;
-  GFile *dir;
-  gchar *module;
-  GFile *module_file;
+  g_autoptr (GFile) file = NULL;
+  g_autoptr (GFile) dir = NULL;
+  g_autofree gchar *module = NULL;
+  g_autoptr (GFile) module_file = NULL;
   GError *tmp_error = NULL;
 
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
 
   file = g_file_new_for_path (self->filename);
   dir = g_file_get_parent (file);
-  g_object_unref (file);
   if (dir == NULL)
     return NULL;
 
   module = retro_core_descriptor_get_module (self, &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
     g_propagate_error (error, tmp_error);
-    g_object_unref (dir);
 
     return NULL;
   }
 
   module_file = g_file_get_child (dir, module);
-  if (!g_file_query_exists (module_file, NULL)) {
-    g_object_unref (dir);
-    g_free (module);
-    g_object_unref (module_file);
-
+  if (!g_file_query_exists (module_file, NULL))
     return NULL;
-  }
 
-  g_object_unref (dir);
-  g_free (module);
-
-  return module_file;
+  return g_steal_pointer (&module_file);
 }
 
 /**
@@ -649,8 +622,10 @@ gboolean
 retro_core_descriptor_has_platform (RetroCoreDescriptor *self,
                                     const gchar         *platform)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
+
   return has_group_prefixed (self,
-                           RETRO_CORE_DESCRIPTOR_PLATFORM_GROUP_PREFIX,
+                           PLATFORM_GROUP_PREFIX,
                            platform);
 }
 
@@ -669,10 +644,12 @@ retro_core_descriptor_has_firmwares (RetroCoreDescriptor  *self,
                                      const gchar          *platform,
                                      GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
+
   return has_key_prefixed (self,
-                           RETRO_CORE_DESCRIPTOR_PLATFORM_GROUP_PREFIX,
+                           PLATFORM_GROUP_PREFIX,
                            platform,
-                           RETRO_CORE_DESCRIPTOR_PLATFORM_FIRMWARES_KEY,
+                           PLATFORM_FIRMWARES_KEY,
                            error);
 }
 
@@ -691,10 +668,12 @@ retro_core_descriptor_has_firmware_md5 (RetroCoreDescriptor  *self,
                                         const gchar          *firmware,
                                         GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
+
   return has_key_prefixed (self,
-                           RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX,
+                           FIRMWARE_GROUP_PREFIX,
                            firmware,
-                           RETRO_CORE_DESCRIPTOR_FIRMWARE_MD5_KEY,
+                           FIRMWARE_MD5_KEY,
                            error);
 }
 
@@ -713,10 +692,12 @@ retro_core_descriptor_has_firmware_sha512 (RetroCoreDescriptor  *self,
                                            const gchar          *firmware,
                                            GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
+
   return has_key_prefixed (self,
-                           RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX,
+                           FIRMWARE_GROUP_PREFIX,
                            firmware,
-                           RETRO_CORE_DESCRIPTOR_FIRMWARE_SHA512_KEY,
+                           FIRMWARE_SHA512_KEY,
                            error);
 }
 
@@ -740,10 +721,12 @@ retro_core_descriptor_get_mime_type (RetroCoreDescriptor  *self,
                                      gsize                *length,
                                      GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
+
   return get_string_list_prefixed (self,
-                                   RETRO_CORE_DESCRIPTOR_PLATFORM_GROUP_PREFIX,
+                                   PLATFORM_GROUP_PREFIX,
                                    platform,
-                                   RETRO_CORE_DESCRIPTOR_PLATFORM_MIME_TYPE_KEY,
+                                   PLATFORM_MIME_TYPE_KEY,
                                    length,
                                    error);
 }
@@ -768,10 +751,12 @@ retro_core_descriptor_get_firmwares (RetroCoreDescriptor  *self,
                                      gsize                *length,
                                      GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
+
   return get_string_list_prefixed (self,
-                                   RETRO_CORE_DESCRIPTOR_PLATFORM_GROUP_PREFIX,
+                                   PLATFORM_GROUP_PREFIX,
                                    platform,
-                                   RETRO_CORE_DESCRIPTOR_PLATFORM_FIRMWARES_KEY,
+                                   PLATFORM_FIRMWARES_KEY,
                                    length,
                                    error);
 }
@@ -791,10 +776,12 @@ retro_core_descriptor_get_firmware_path (RetroCoreDescriptor  *self,
                                          const gchar          *firmware,
                                          GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
+
   return get_string_prefixed (self,
-                              RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX,
+                              FIRMWARE_GROUP_PREFIX,
                               firmware,
-                              RETRO_CORE_DESCRIPTOR_FIRMWARE_PATH_KEY,
+                              FIRMWARE_PATH_KEY,
                               error);
 }
 
@@ -813,10 +800,12 @@ retro_core_descriptor_get_firmware_md5 (RetroCoreDescriptor  *self,
                                         const gchar          *firmware,
                                         GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
+
   return get_string_prefixed (self,
-                              RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX,
+                              FIRMWARE_GROUP_PREFIX,
                               firmware,
-                              RETRO_CORE_DESCRIPTOR_FIRMWARE_MD5_KEY,
+                              FIRMWARE_MD5_KEY,
                               error);
 }
 
@@ -835,10 +824,12 @@ retro_core_descriptor_get_firmware_sha512 (RetroCoreDescriptor  *self,
                                            const gchar          *firmware,
                                            GError              **error)
 {
+  g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), NULL);
+
   return get_string_prefixed (self,
-                              RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX,
+                              FIRMWARE_GROUP_PREFIX,
                               firmware,
-                              RETRO_CORE_DESCRIPTOR_FIRMWARE_SHA512_KEY,
+                              FIRMWARE_SHA512_KEY,
                               error);
 }
 
@@ -857,28 +848,25 @@ retro_core_descriptor_get_is_firmware_mandatory (RetroCoreDescriptor  *self,
                                                  const gchar          *firmware,
                                                  GError              **error)
 {
-  gchar *group;
-  gboolean result;
+  g_autofree gchar *group = NULL;
+  gboolean is_mandatory;
   GError *tmp_error = NULL;
 
   g_return_val_if_fail (RETRO_IS_CORE_DESCRIPTOR (self), FALSE);
   g_return_val_if_fail (firmware != NULL, FALSE);
 
-  group = g_strconcat (RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX, firmware, NULL);
-  result = g_key_file_get_boolean (self->key_file,
-                                   group,
-                                   RETRO_CORE_DESCRIPTOR_FIRMWARE_MANDATORY_KEY,
-                                   &tmp_error);
+  group = g_strconcat (FIRMWARE_GROUP_PREFIX, firmware, NULL);
+  is_mandatory = g_key_file_get_boolean (self->key_file,
+                                         group,
+                                         FIRMWARE_MANDATORY_KEY,
+                                         &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
-    g_free (group);
     g_propagate_error (error, tmp_error);
 
     return FALSE;
   }
 
-  g_free (group);
-
-  return result;
+  return is_mandatory;
 }
 
 /**
@@ -898,7 +886,7 @@ retro_core_descriptor_get_platform_supports_mime_types (RetroCoreDescriptor  *se
                                                         const gchar * const  *mime_types,
                                                         GError              **error)
 {
-  gchar **supported_mime_types;
+  g_auto (GStrv) supported_mime_types = NULL;
   gsize supported_mime_types_length;
   GError *tmp_error = NULL;
 
@@ -919,13 +907,8 @@ retro_core_descriptor_get_platform_supports_mime_types (RetroCoreDescriptor  *se
 
   for (; *mime_types != NULL; mime_types++)
     if (!g_strv_contains ((const char * const *) supported_mime_types,
-                          *mime_types)) {
-      g_strfreev (supported_mime_types);
-
+                          *mime_types))
       return FALSE;
-    }
-
-  g_strfreev (supported_mime_types);
 
   return TRUE;
 }
@@ -943,10 +926,9 @@ RetroCoreDescriptor *
 retro_core_descriptor_new (const gchar  *filename,
                            GError      **error)
 {
-  RetroCoreDescriptor *self;
-  gchar ** groups;
+  g_autoptr (RetroCoreDescriptor) self = NULL;
+  g_auto (GStrv) groups = NULL;
   gsize groups_length;
-  gsize i;
   GError *tmp_error = NULL;
 
   g_return_val_if_fail (filename != NULL, NULL);
@@ -959,7 +941,6 @@ retro_core_descriptor_new (const gchar  *filename,
                              G_KEY_FILE_NONE,
                              &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
-    g_object_unref (self);
     g_propagate_error (error, tmp_error);
 
     return NULL;
@@ -967,31 +948,26 @@ retro_core_descriptor_new (const gchar  *filename,
 
   check_libretro_group (self, &tmp_error);
   if (G_UNLIKELY (tmp_error != NULL)) {
-    g_object_unref (self);
     g_propagate_error (error, tmp_error);
 
     return NULL;
   }
 
   groups = g_key_file_get_groups (self->key_file, &groups_length);
-  for (i = 0; i < groups_length; i++) {
+  for (gsize i = 0; i < groups_length; i++) {
     if (g_str_has_prefix (groups[i],
-                          RETRO_CORE_DESCRIPTOR_PLATFORM_GROUP_PREFIX)) {
+                          PLATFORM_GROUP_PREFIX)) {
       check_platform_group (self, groups[i], &tmp_error);
       if (G_UNLIKELY (tmp_error != NULL)) {
-        g_strfreev (groups);
-        g_object_unref (self);
         g_propagate_error (error, tmp_error);
 
         return NULL;
       }
     }
     else if (g_str_has_prefix (groups[i],
-                               RETRO_CORE_DESCRIPTOR_FIRMWARE_GROUP_PREFIX)) {
+                               FIRMWARE_GROUP_PREFIX)) {
       check_firmware_group (self, groups[i], &tmp_error);
       if (G_UNLIKELY (tmp_error != NULL)) {
-        g_strfreev (groups);
-        g_object_unref (self);
         g_propagate_error (error, tmp_error);
 
         return NULL;
@@ -999,7 +975,5 @@ retro_core_descriptor_new (const gchar  *filename,
     }
   }
 
-  g_strfreev (groups);
-
-  return self;
+  return g_steal_pointer (&self);
 }
