@@ -1,7 +1,6 @@
 // This file is part of retro-gtk. License: GPL-3.0+.
 
-#ifndef RETRO_LOG_H
-#define RETRO_LOG_H
+#pragma once
 
 #if !defined(__RETRO_GTK_INSIDE__) && !defined(RETRO_GTK_COMPILATION)
 # error "Only <retro-gtk.h> can be included directly."
@@ -18,5 +17,3 @@ void retro_g_log (RetroCore      *sender,
                   const gchar    *message);
 
 G_END_DECLS
-
-#endif /* RETRO_LOG_H */
