@@ -1,7 +1,6 @@
 // This file is part of retro-gtk. License: GPL-3.0+.
 
-#ifndef RETRO_PIXDATA_PRIVATE_H
-#define RETRO_PIXDATA_PRIVATE_H
+#pragma once
 
 #if !defined(__RETRO_GTK_INSIDE__) && !defined(RETRO_GTK_COMPILATION)
 # error "Only <retro-gtk.h> can be included directly."
@@ -9,7 +8,7 @@
 
 #include "retro-pixdata.h"
 
-#include "retro-pixel-format.h"
+#include "retro-pixel-format-private.h"
 
 G_BEGIN_DECLS
 
@@ -32,5 +31,3 @@ void retro_pixdata_init (RetroPixdata     *self,
                          gfloat            aspect_ratio);
 
 G_END_DECLS
-
-#endif /* RETRO_PIXDATA_PRIVATE_H */
