@@ -137,27 +137,18 @@ rgba8888_from_video (gconstpointer  src,
                      gsize          pitch,
                      GetRGBA8888    get_pixel)
 {
-  gsize row, src_row, dst_row, col, src_col;
+  for (gsize row = 0 ; row < height ; row++) {
+    gsize src_row = row * pitch;
+    gsize dst_row = row * width;
 
-  for (row = 0 ; row < height ; row++) {
-    src_row = row * pitch;
-    dst_row = row * width;
-
-    for (col = 0 ; col < width ; col++) {
-      src_col = col * pixel_size;
+    for (gsize col = 0 ; col < width ; col++) {
+      gsize src_col = col * pixel_size;
 
       dst[dst_row + col] = get_pixel (src_row + src_col + src);
     }
   }
 }
 
-static void
-pixels_free (guchar   *pixels,
-             gpointer  data)
-{
-  g_free (pixels);
-}
-
 /**
  * retro_pixdata_new:
  * @data: the video data
@@ -360,7 +351,7 @@ retro_pixdata_to_pixbuf (RetroPixdata *self)
                                      GDK_COLORSPACE_RGB, TRUE, 8,
                                      self->width, self->height,
                                      self->width * sizeof (rgba8888),
-                                     pixels_free, NULL);
+                                     (GdkPixbufDestroyNotify) g_free, NULL);
 
   /* x-dpi and y-dpi are deprecated, retro_pixbuf_get_aspect_ratio() and
    * retro_pixbuf_set_aspect_ratio() should be used instead. */
