@@ -21,9 +21,9 @@
 RetroVideoFilter
 retro_video_filter_from_string (const gchar *filter)
 {
-  GEnumClass* enum_class;
+  g_autoptr (GEnumClass) enum_class = NULL;
+  GEnumValue* eval;
   RetroVideoFilter result;
-  GEnumValue* eval = NULL;
 
   g_return_val_if_fail (filter != NULL, RETRO_VIDEO_FILTER_SMOOTH);
 
@@ -34,7 +34,5 @@ retro_video_filter_from_string (const gchar *filter)
     RETRO_VIDEO_FILTER_SMOOTH :
     (RetroVideoFilter) eval->value;
 
-  g_type_class_unref (enum_class);
-
   return result;
 }
