@@ -116,13 +116,11 @@ retro_core_finalize (GObject *object)
   deinit = retro_module_get_deinit (self->module);
   deinit ();
 
-  if (self->media_uris != NULL)
-    g_strfreev (self->media_uris);
+  g_clear_pointer (&self->media_uris, g_strfreev);
 
   g_object_unref (self->module);
   g_object_unref (self->framebuffer);
-  if (self->default_controller)
-    g_object_unref (self->default_controller);
+  g_clear_object (&self->default_controller);
   g_hash_table_unref (self->controllers);
   g_hash_table_unref (self->variables);
   g_hash_table_unref (self->variable_overrides);
@@ -820,7 +818,6 @@ load_discs (RetroCore  *self,
 {
   guint length;
   gboolean fullpath;
-  guint index;
   GError *tmp_error = NULL;
 
   set_disk_ejected (self, TRUE, &tmp_error);
@@ -848,7 +845,7 @@ load_discs (RetroCore  *self,
   }
 
   fullpath = get_needs_full_path (self);
-  for (index = 0; index < length; index++) {
+  for (gsize index = 0; index < length; index++) {
     g_autoptr (GFile) file = g_file_new_for_uri (self->media_uris[index]);
     g_autofree gchar *path = g_file_get_path (file);
     g_autoptr (RetroGameInfo) game_info = fullpath ?
@@ -1238,7 +1235,7 @@ retro_core_get_support_no_game (RetroCore *self)
 /**
  * retro_core_set_support_no_game:
  * @self: a #RetroCore
- * @support_no_game: the save directory
+ * @support_no_game: whether the core supports running with no game
  *
  * Sets whether the core supports running with no game.
  */
@@ -1248,6 +1245,8 @@ retro_core_set_support_no_game (RetroCore *self,
 {
   g_return_if_fail (RETRO_IS_CORE (self));
 
+  support_no_game = !!support_no_game;
+
   if (retro_core_get_support_no_game (self) == support_no_game)
     return;
 
@@ -1320,9 +1319,7 @@ retro_core_set_medias (RetroCore           *self,
   g_return_if_fail (RETRO_IS_CORE (self));
   g_return_if_fail (!retro_core_get_is_initiated (self));
 
-  if (self->media_uris != NULL)
-    g_strfreev (self->media_uris);
-
+  g_clear_pointer (&self->media_uris, g_strfreev);
   self->media_uris = g_strdupv ((gchar **) uris);
 }
 
