@@ -3,11 +3,17 @@
 #include "retro-core-private.h"
 
 #include <stdbool.h>
+#include "retro-debug-private.h"
 #include "retro-input-private.h"
 #include "retro-gl-renderer-private.h"
 #include "retro-hw-render-callback-private.h"
 #include "retro-rumble-effect.h"
 
+#undef G_LOG_DOMAIN
+#define G_LOG_DOMAIN "RetroEnvironment"
+
+#define TRUENESS(boolean) ((boolean) ? "true" : "false")
+
 #define RETRO_ENVIRONMENT_EXPERIMENTAL 0x10000
 #define RETRO_ENVIRONMENT_PRIVATE 0x20000
 #define RETRO_ENVIRONMENT_SET_ROTATION 1
@@ -136,7 +142,7 @@ log_cb (guint level, const gchar *format, ...)
 
     break;
   default:
-    g_debug ("Unexpected log level: %d", level);
+    retro_debug ("Unexpected log level: %d", level);
 
     return;
   }
@@ -150,14 +156,68 @@ log_cb (guint level, const gchar *format, ...)
   g_signal_emit_by_name (self, "log", log_domain, log_level, message);
 }
 
+static const gchar *
+get_input_string (RetroInputDescriptor *descriptor) {
+  const gchar *controller_type_name = NULL;
+  g_autoptr (GEnumClass) index_enum_class = NULL;
+  g_autoptr (GEnumClass) id_enum_class = NULL;
+  GEnumValue *id_enum_value;
+
+  switch (descriptor->controller_type & RETRO_CONTROLLER_TYPE_TYPE_MASK) {
+  case RETRO_CONTROLLER_TYPE_JOYPAD:
+    controller_type_name = "joypad";
+    id_enum_class = g_type_class_ref (RETRO_TYPE_JOYPAD_ID);
+    break;
+  case RETRO_CONTROLLER_TYPE_MOUSE:
+    controller_type_name = "mouse";
+    id_enum_class = g_type_class_ref (RETRO_TYPE_MOUSE_ID);
+    break;
+  case RETRO_CONTROLLER_TYPE_LIGHTGUN:
+    controller_type_name = "lightgun";
+    id_enum_class = g_type_class_ref (RETRO_TYPE_LIGHTGUN_ID);
+    break;
+  case RETRO_CONTROLLER_TYPE_ANALOG:
+    controller_type_name = "analog";
+    index_enum_class = g_type_class_ref (RETRO_TYPE_ANALOG_INDEX);
+    id_enum_class = g_type_class_ref (RETRO_TYPE_ANALOG_ID);
+    break;
+  case RETRO_CONTROLLER_TYPE_POINTER:
+    controller_type_name = "pointer";
+    id_enum_class = g_type_class_ref (RETRO_TYPE_POINTER_ID);
+    break;
+  case RETRO_CONTROLLER_TYPE_KEYBOARD:
+    return g_strdup ("keyboard key");
+  case RETRO_CONTROLLER_TYPE_NONE:
+    return g_strdup ("none");
+  default:
+    return g_strdup ("unknown input");
+  }
+
+  id_enum_value = g_enum_get_value (id_enum_class, descriptor->id);
+
+  if (index_enum_class) {
+    GEnumValue *index_enum_value = g_enum_get_value (index_enum_class, descriptor->index);
+
+    return (index_enum_value && id_enum_value) ?
+      g_strdup_printf ("%s %s %s", controller_type_name, index_enum_value->value_nick, id_enum_value->value_nick) :
+      g_strdup_printf ("unknown %s input", controller_type_name);
+  }
+
+  return (id_enum_value) ?
+    g_strdup_printf ("%s %s", controller_type_name, id_enum_value->value_nick) :
+    g_strdup_printf ("unknown %s input", controller_type_name);
+}
+
 /* Environment commands */
 
 static gboolean
 get_can_dupe (RetroCore *self,
-              gboolean  *can_dupe)
+              bool      *can_dupe)
 {
   *can_dupe = TRUE;
 
+  retro_debug ("Get can dupe: true");
+
   return TRUE;
 }
 
@@ -167,6 +227,8 @@ get_content_directory (RetroCore    *self,
 {
   *(content_directory) = retro_core_get_content_directory (self);
 
+  retro_debug ("Get content directory: %s", *content_directory);
+
   return TRUE;
 }
 
@@ -176,6 +238,8 @@ get_input_device_capabilities (RetroCore *self,
 {
   *capabilities = retro_core_get_controller_capabilities (self);
 
+  retro_debug ("Get input device capabilities");
+
   return TRUE;
 }
 
@@ -207,9 +271,9 @@ get_language (RetroCore *self,
   };
 
   const gchar * const *locales = g_get_language_names ();
-  gsize locale_i, language_i = 0;
+  gsize language_i = 0;
 
-  for (locale_i = 0; locales[locale_i] != NULL; locale_i++) {
+  for (gsize locale_i = 0; locales[locale_i] != NULL; locale_i++) {
     for (language_i = 0;
          !g_str_equal (values[language_i].locale, "C") &&
          !g_str_equal (locales[locale_i], values[language_i].locale);
@@ -220,6 +284,8 @@ get_language (RetroCore *self,
 
   *language = values[language_i].language;
 
+  retro_debug ("Get language: %s", values[language_i].locale);
+
   return TRUE;
 }
 
@@ -229,6 +295,8 @@ get_libretro_path (RetroCore    *self,
 {
   *(libretro_directory) = retro_core_get_libretro_path (self);
 
+  retro_debug ("Get libretro directory: %s", *libretro_directory);
+
   return TRUE;
 }
 
@@ -238,15 +306,19 @@ get_log_callback (RetroCore        *self,
 {
   cb->log = log_cb;
 
+  retro_debug ("Get log callback");
+
   return TRUE;
 }
 
 static gboolean
 get_overscan (RetroCore *self,
-              gboolean  *overscan)
+              bool      *overscan)
 {
   *overscan = self->overscan;
 
+  retro_debug ("Get overscan: %s", TRUENESS (*overscan));
+
   return TRUE;
 }
 
@@ -256,6 +328,8 @@ get_rumble_callback (RetroCore           *self,
 {
   cb->set_rumble_state = rumble_callback_set_rumble_state;
 
+  retro_debug ("Get rumble callback");
+
   return TRUE;
 }
 
@@ -265,6 +339,8 @@ get_save_directory (RetroCore    *self,
 {
   *(save_directory) = retro_core_get_save_directory (self);
 
+  retro_debug ("Get save directory: %s", *save_directory);
+
   return TRUE;
 }
 
@@ -274,6 +350,8 @@ get_system_directory (RetroCore    *self,
 {
   *(system_directory) = retro_core_get_system_directory (self);
 
+  retro_debug ("Get system directory: %s", *system_directory);
+
   return TRUE;
 }
 
@@ -285,11 +363,16 @@ get_variable (RetroCore     *self,
 
   value = g_hash_table_lookup (self->variables, variable->key);
 
-  if (!value)
+  if (G_UNLIKELY (!value)) {
+    g_critical ("Couldn't get variable %s", variable->key);
+
     return FALSE;
+  }
 
   variable->value = value;
 
+  retro_debug ("Get variable %s: %s", variable->key, variable->value);
+
   return TRUE;
 }
 
@@ -300,6 +383,8 @@ get_variable_update (RetroCore *self,
 {
   *update = retro_core_get_variable_update (self);
 
+  retro_debug ("Get variable update: %s", TRUENESS (*update));
+
   return TRUE;
 }
 
@@ -329,15 +414,21 @@ set_hw_render (RetroCore             *self,
   case RETRO_HW_CONTEXT_OPENGLES2:
   case RETRO_HW_CONTEXT_OPENGLES3:
   case RETRO_HW_CONTEXT_OPENGLES_VERSION:
+    retro_debug ("Set hardware render callback: OpenGL");
+
     self->renderer = retro_gl_renderer_new (self, callback);
     break;
 
   case RETRO_HW_CONTEXT_VULKAN:
-    g_critical ("Vulkan support isn't implemented");
+    g_critical ("Couldn't set hardware render callback: Vulkan support is unimplemented");
+
+    return FALSE;
+  case RETRO_HW_CONTEXT_DIRECT3D:
+    g_critical ("Couldn't set hardware render callback: Direct3D is unsupported");
 
     return FALSE;
   default:
-    g_critical ("Unknown context type: %d", callback->context_type);
+    g_critical ("Couldn't set hardware render callback for unknown context type %d", callback->context_type);
 
     return FALSE;
   }
@@ -352,6 +443,8 @@ static gboolean
 set_disk_control_interface (RetroCore                *self,
                             RetroDiskControlCallback *callback)
 {
+  retro_debug ("Set disk control callback");
+
   self->disk_control_callback = callback;
 
   return TRUE;
@@ -361,6 +454,11 @@ static gboolean
 set_geometry (RetroCore         *self,
               RetroGameGeometry *geometry)
 {
+  retro_debug ("Set geometry: base %u × %u, max %u × %u, aspect ratio %f",
+               geometry->base_width, geometry->base_height,
+               geometry->max_width, geometry->max_height,
+               geometry->aspect_ratio);
+
   retro_core_set_geometry (self, geometry);
 
   return TRUE;
@@ -372,7 +470,16 @@ set_input_descriptors (RetroCore            *self,
 {
   int length;
 
-  for (length = 0 ; descriptors[length].description ; length++);
+  for (length = 0 ; descriptors[length].description ; length++)
+    retro_debug ("Set input descriptor: port %u, type %u, index %u, id %u (%s%s): %s",
+                 descriptors[length].port,
+                 descriptors[length].controller_type,
+                 descriptors[length].index,
+                 descriptors[length].id,
+                 get_input_string (&descriptors[length]),
+                 (descriptors[length].controller_type & ~RETRO_CONTROLLER_TYPE_TYPE_MASK) ? ", specialized" : "",
+                 descriptors[length].description);
+
   retro_core_set_controller_descriptors (self, descriptors, length);
 
   return TRUE;
@@ -382,6 +489,8 @@ static gboolean
 set_keyboard_callback (RetroCore             *self,
                        RetroKeyboardCallback *callback)
 {
+  retro_debug ("Set keyboard callback");
+
   self->keyboard_callback = *callback;
 
   return TRUE;
@@ -391,6 +500,8 @@ static gboolean
 set_message (RetroCore          *self,
              const RetroMessage *message)
 {
+  retro_debug ("Emit message for %u frames: %s", message->frames, message->msg);
+
   g_signal_emit_by_name (self, "message", message->msg, message->frames);
 
   return TRUE;
@@ -400,6 +511,25 @@ static gboolean
 set_pixel_format (RetroCore              *self,
                   const RetroPixelFormat *pixel_format)
 {
+  switch (*pixel_format) {
+  case RETRO_PIXEL_FORMAT_XRGB1555:
+    retro_debug ("Set pixel format: XRGB1555");
+
+    break;
+  case RETRO_PIXEL_FORMAT_XRGB8888:
+    retro_debug ("Set pixel format: XRGB8888");
+
+    break;
+  case RETRO_PIXEL_FORMAT_RGB565:
+    retro_debug ("Set pixel format: RGB565");
+
+    break;
+  default:
+    g_critical ("Couldn't set unknown pixel format %d", *pixel_format);
+
+    return FALSE;
+  }
+
   self->pixel_format = *pixel_format;
 
   return TRUE;
@@ -409,15 +539,25 @@ static gboolean
 set_rotation (RetroCore           *self,
               const RetroRotation *rotation)
 {
+  if (G_UNLIKELY (*rotation >= CLOCKWISE)) {
+    g_critical ("Couldn't set unknown rotation %d", *rotation);
+
+    return FALSE;
+  }
+
+  retro_debug ("Set rotation: %d°", *rotation * 90);
+
   self->rotation = *rotation;
 
   return TRUE;
 }
 
 static gboolean
-set_support_no_game (RetroCore *self,
-                     gboolean  *support_no_game)
+set_support_no_game (RetroCore  *self,
+                     const bool *support_no_game)
 {
+  retro_debug ("Set support no game: %s", TRUENESS (*support_no_game));
+
   retro_core_set_support_no_game (self, *support_no_game);
 
   return TRUE;
@@ -427,6 +567,12 @@ static gboolean
 set_system_av_info (RetroCore         *self,
                     RetroSystemAvInfo *system_av_info)
 {
+  retro_debug ("Set system AV info: base %u × %u, max %u × %u, aspect ratio %f, %lf FPS, %lf Hz",
+               system_av_info->geometry.base_width, system_av_info->geometry.base_height,
+               system_av_info->geometry.max_width, system_av_info->geometry.max_height,
+               system_av_info->geometry.aspect_ratio,
+               system_av_info->timing.fps, system_av_info->timing.sample_rate);
+
   retro_core_set_system_av_info (self, system_av_info);
 
   return TRUE;
@@ -436,10 +582,11 @@ static gboolean
 set_variables (RetroCore     *self,
                RetroVariable *variable_array)
 {
-  int i;
+  for (gsize i = 0 ; variable_array[i].key && variable_array[i].value ; i++) {
+    retro_debug ("Set variable %s: %s", variable_array[i].key, variable_array[i].value);
 
-  for (i = 0 ; variable_array[i].key && variable_array[i].value ; i++)
     retro_core_insert_variable (self, &variable_array[i]);
+  }
 
   g_signal_emit_by_name (self, "variables-set", variable_array);
 
@@ -449,11 +596,23 @@ set_variables (RetroCore     *self,
 static gboolean
 shutdown (RetroCore *self)
 {
+  retro_debug ("Emit shutdown");
+
   g_signal_emit_by_name (self, "shutdown");
 
   return TRUE;
 }
 
+#define RETRO_UNIMPLEMENT_ENVIRONMENT(cmd) \
+  case cmd: \
+    G_STMT_START { \
+      if (cmd & RETRO_ENVIRONMENT_EXPERIMENTAL) \
+        g_critical ("Unimplemented experimental command %s", #cmd); \
+      else \
+        g_critical ("Unimplemented command %s", #cmd); \
+      return FALSE; \
+    } G_STMT_END
+
 static gboolean
 environment_core_command (RetroCore *self,
                           unsigned   cmd,
@@ -464,7 +623,7 @@ environment_core_command (RetroCore *self,
 
   switch (cmd) {
   case RETRO_ENVIRONMENT_GET_CAN_DUPE:
-    return get_can_dupe (self, (gboolean *) data);
+    return get_can_dupe (self, (bool *) data);
 
   case RETRO_ENVIRONMENT_GET_CONTENT_DIRECTORY:
     return get_content_directory (self, (const gchar **) data);
@@ -482,7 +641,7 @@ environment_core_command (RetroCore *self,
     return get_log_callback (self, (RetroLogCallback *) data);
 
   case RETRO_ENVIRONMENT_GET_OVERSCAN:
-    return get_overscan (self, (gboolean *) data);
+    return get_overscan (self, (bool *) data);
 
   case RETRO_ENVIRONMENT_GET_RUMBLE_INTERFACE:
     return get_rumble_callback (self, (RetroRumbleCallback *) data);
@@ -524,7 +683,7 @@ environment_core_command (RetroCore *self,
     return set_rotation (self, (RetroRotation *) data);
 
   case RETRO_ENVIRONMENT_SET_SUPPORT_NO_GAME:
-    return set_support_no_game (self, (gboolean *) data);
+    return set_support_no_game (self, (const bool *) data);
 
   case RETRO_ENVIRONMENT_SET_SYSTEM_AV_INFO:
     return set_system_av_info (self, (RetroSystemAvInfo *) data);
@@ -535,24 +694,30 @@ environment_core_command (RetroCore *self,
   case RETRO_ENVIRONMENT_SHUTDOWN:
     return shutdown (self);
 
-  case RETRO_ENVIRONMENT_GET_CAMERA_INTERFACE:
-  case RETRO_ENVIRONMENT_GET_CURRENT_SOFTWARE_FRAMEBUFFER:
-  case RETRO_ENVIRONMENT_GET_HW_RENDER_INTERFACE:
-  case RETRO_ENVIRONMENT_GET_LOCATION_INTERFACE:
-  case RETRO_ENVIRONMENT_GET_PERF_INTERFACE:
-  case RETRO_ENVIRONMENT_GET_SENSOR_INTERFACE:
-  case RETRO_ENVIRONMENT_GET_USERNAME:
-  case RETRO_ENVIRONMENT_SET_AUDIO_CALLBACK:
-  case RETRO_ENVIRONMENT_SET_CONTROLLER_INFO:
-  case RETRO_ENVIRONMENT_SET_FRAME_TIME_CALLBACK:
-  case RETRO_ENVIRONMENT_SET_HW_RENDER_CONTEXT_NEGOTIATION_INTERFACE:
-  case RETRO_ENVIRONMENT_SET_MEMORY_MAPS:
-  case RETRO_ENVIRONMENT_SET_PERFORMANCE_LEVEL:
-  case RETRO_ENVIRONMENT_SET_PROC_ADDRESS_CALLBACK:
-  case RETRO_ENVIRONMENT_SET_SERIALIZATION_QUIRKS:
-  case RETRO_ENVIRONMENT_SET_SUBSYSTEM_INFO:
-  case RETRO_ENVIRONMENT_SET_SUPPORT_ACHIEVEMENTS:
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_CAMERA_INTERFACE);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_CURRENT_SOFTWARE_FRAMEBUFFER);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_HW_RENDER_INTERFACE);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_LOCATION_INTERFACE);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_PERF_INTERFACE);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_SENSOR_INTERFACE);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_GET_USERNAME);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_AUDIO_CALLBACK);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_CONTROLLER_INFO);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_FRAME_TIME_CALLBACK);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_HW_RENDER_CONTEXT_NEGOTIATION_INTERFACE);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_MEMORY_MAPS);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_PERFORMANCE_LEVEL);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_PROC_ADDRESS_CALLBACK);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_SERIALIZATION_QUIRKS);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_SUBSYSTEM_INFO);
+  RETRO_UNIMPLEMENT_ENVIRONMENT (RETRO_ENVIRONMENT_SET_SUPPORT_ACHIEVEMENTS);
+
   default:
+    if (cmd & RETRO_ENVIRONMENT_EXPERIMENTAL)
+      g_critical ("Unknown experimental command %d", cmd ^ RETRO_ENVIRONMENT_EXPERIMENTAL);
+    else
+      g_critical ("Unknown command %d", cmd);
+
     return FALSE;
   }
 }
@@ -587,7 +752,7 @@ video_refresh_cb (guint8 *data,
   if (self->renderer) {
     gint pixel_size;
 
-    if (data && data != RETRO_HW_FRAME_BUFFER_VALID) {
+    if (G_UNLIKELY (data && data != RETRO_HW_FRAME_BUFFER_VALID)) {
       g_critical ("Video data must be NULL or RETRO_HW_FRAME_BUFFER_VALID if "
                   "rendering to hardware.");
 
