@@ -14,7 +14,7 @@ G_BEGIN_DECLS
 #define RETRO_ANALOG_INDEX_COUNT (RETRO_ANALOG_INDEX_RIGHT + 1)
 #define RETRO_JOYPAD_ID_COUNT (RETRO_JOYPAD_ID_R3 + 1)
 #define RETRO_LIGHTGUN_ID_COUNT (RETRO_LIGHTGUN_ID_START + 1)
-#define RETRO_MOUSE_ID_COUNT (RETRO_MOUSE_ID_HORIZ_WHEELDOWN + 1)
+#define RETRO_MOUSE_ID_COUNT (RETRO_MOUSE_ID_BUTTON_5 + 1)
 #define RETRO_POINTER_ID_COUNT (RETRO_POINTER_ID_PRESSED + 1)
 
 G_END_DECLS
